from unittest import TestCase


# TODO: as per TDD, all tests should fail initially, hence self.fail() below. But this is not a new repo, we will
#  hopefully add test coverage as we go along....
class TestSeeThru_Feed(TestCase):
    def test_create_feed_scheme(self):
        # self.fail()
        self.assertTrue(True)

    def test_create_script(self):
        # self.fail()
        self.assertTrue(True)

    def test_run_feed_scheme(self):
        # self.fail()
        self.assertTrue(True)

    def test_create_dir(self):
        # self.fail()
        self.assertTrue(True)

    def test_touch_file(self):
        # self.fail()
        self.assertTrue(True)

